#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <dirent.h>
#include <unistd.h>

#include "disk_info.h"

static File *traverse_file(char *);
static Dir  *traverse_dir (char *);


static
File *traverse_file(char *filename){
    File        *file = NULL;
    struct stat sb;

    file = calloc(sizeof(File),sizeof(char));
    assert(file);
    lstat(filename,&sb);

    file->file_type = IS_FILE;
    strcpy(file->file_name,filename);
    file->file_date = sb.st_mtime;
    file->file_size = sb.st_size;

    return file;
}

static
Dir  *traverse_dir(char *dirname){
    Dir         *dir  = NULL;
    Dir         *ndir = NULL;
    File        *file = NULL;
    DIR         *dirfd;
    struct dirent *de;

    dir = calloc(sizeof(Dir),sizeof(char));
    assert(dir);

    strcpy(dir->file_name,dirname);

    dir->file_type = IS_DIRECTORY;

    dirfd = opendir(".");
    assert(dirfd);

    while((de = readdir(dirfd))){
        if(de->d_name[0] == '.' && de->d_name[1] == '\0')
            continue;
        if(de->d_name[0] == '.' && de->d_name[1] == '.')
            continue;

        if(de->d_type & DT_REG){
            file = traverse_file(de->d_name);
            dir->dir_content = g_slist_append(dir->dir_content,file);
            dir->file_size+= file->file_size;
        }
        else if(de->d_type & DT_DIR){
            if(chdir(de->d_name)<0)
                perror("chdir");
            ndir = traverse_dir(de->d_name);
            dir->dir_content = g_slist_append(dir->dir_content,ndir);
            dir->file_size+= ndir->file_size;
            if(chdir("..")<0)
                perror("chdir");
        }
    }
    free(de);
    closedir(dirfd);
    return dir;
}

Disk *traverse_disk(char *dirname){
    Disk        *disk;
    File        *file;
    Dir         *dir;
    DIR         *dirfd;
    struct dirent *de;
    char        dirini[BUFSIZ];


    if((dirfd = opendir(dirname))==NULL){
        perror("opendir");
        return NULL;
    }

    if((disk = (Disk*)calloc(sizeof(Disk),sizeof(char)))==NULL){
        fprintf(stderr,"Malloc failed %s:%d\n", __FILE__,__LINE__);
        return NULL;
    }

    getwd((char*)dirini);
    if(chdir(dirname)<0){
        perror("chdir");
        return NULL;
    }

    strcpy(disk->disk_name,dirname);
    time(&disk->disk_date);

    while((de = readdir(dirfd))){
        
        if(de->d_name[0] == '.' && de->d_name[1] == '\0')
            continue;
        if(de->d_name[0] == '.' && de->d_name[1] == '.')
            continue;

        if(de->d_type & DT_REG){
            file = traverse_file(de->d_name); 
            disk->disk_content = g_slist_append(disk->disk_content,file);
            disk->disk_size+=file->file_size;
        }
        else if(de->d_type & DT_DIR){
            if(chdir(de->d_name)<0)
                perror("chdir");
            dir  = traverse_dir(de->d_name);
            disk->disk_content = g_slist_append(disk->disk_content,dir);
            disk->disk_size+=dir->file_size;
            if(chdir("..")<0)
                perror("chdir");
        }
    }
    free(de);
    closedir(dirfd);

    chdir(dirini);
    
    return disk;
}
