#include <gtk/gtk.h>

#include "menu_callbacks.h"

static void
activate_action (GtkAction *action)
{
  g_message ("Action \"%s\" activated", gtk_action_get_name (action));
}

static GtkActionEntry menu_entries[] = {
  { "FileMenu", NULL, "_File" },               /* name, stock id, label */
  { "New", GTK_STOCK_NEW,                      /* name, stock id */
    "_New", "<control>N",                      /* label, accelerator */
    "Create a new file",                       /* tooltip */ 
    G_CALLBACK (NULL) },      
  { "Open", GTK_STOCK_OPEN,                    /* name, stock id */
    "_Open","<control>O",                      /* label, accelerator */     
    "Open a file",                             /* tooltip */
    G_CALLBACK (menu_file_open) }, 
  { "Save", GTK_STOCK_SAVE,                    /* name, stock id */
    "_Save","<control>S",                      /* label, accelerator */     
    "Save current file",                       /* tooltip */
    G_CALLBACK (activate_action) },
  { "SaveAs", GTK_STOCK_SAVE,                  /* name, stock id */
    "Save _As...", NULL,                       /* label, accelerator */     
    "Save to a file",                          /* tooltip */
    G_CALLBACK (activate_action) },
  { "Quit", GTK_STOCK_QUIT,                    /* name, stock id */
    "_Quit", "<control>Q",                     /* label, accelerator */     
    "Quit",                                    /* tooltip */
    G_CALLBACK (menu_file_quit) },
  { "Edit", NULL, "_Edit"},
  { "Add" , GTK_STOCK_ADD,
    "_Add a disk", NULL,
    "Add a new disk",
    G_CALLBACK (menu_edit_add) },
  { "Remove" , GTK_STOCK_CANCEL,
    "_Remove a disk", NULL,
    "Remove a disk",
    G_CALLBACK (menu_edit_remove) },
  { "Find", NULL, "_Find"},
  { "Search" , GTK_STOCK_FIND,
    "_Search a disk", NULL,
    "Search a disk",
    G_CALLBACK (NULL) },
 
};
static guint menu_n_entries = G_N_ELEMENTS (menu_entries);


static const gchar *ui_menu = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Save'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='Edit'>"
"     <menuitem action='Add'/>"
"     <menuitem action='Remove'/>"
"    </menu>"
"    <menu action='Find'>"
"     <menuitem action='Search'/>"
"    </menu>"
"  </menubar>"
"  <toolbar  name='ToolBar'>"
"    <toolitem action='Open'/>"
"    <toolitem action='Quit'/>"
"    <toolitem action='Add'/>"
"    <toolitem action='Remove'/>"
"    <toolitem action='Search'/>"
"  </toolbar>"
"</ui>";

void do_menu(GtkWidget *window)
{
    GtkUIManager   *ui          = NULL;
    GtkActionGroup *actions     = NULL;
    GError         *error       = NULL;
    GtkWidget      *box         = NULL;

    box = g_object_get_data (G_OBJECT(window),"win_v_cont");

    actions = gtk_action_group_new ("Actions");
    gtk_action_group_add_actions (actions, menu_entries, menu_n_entries,window);

    ui = gtk_ui_manager_new ();

    gtk_ui_manager_insert_action_group (ui, actions, 0);
    
    /* 
     * Verifier les erreur de parsing 
     */
    if (!gtk_ui_manager_add_ui_from_string (ui, ui_menu, -1, &error)){
        g_message ("building menus failed: %s", error->message);
        g_error_free (error);
    }

    gtk_box_pack_start (GTK_BOX (box),
                        gtk_ui_manager_get_widget (ui, "/MenuBar"),
                        FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box),
                        gtk_ui_manager_get_widget (ui, "/ToolBar"),
                        FALSE, FALSE, 0);
}

