#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <dirent.h>
#include <unistd.h>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include <glib.h>

#include "disk_info.h"


/*
 * Prototypes 
 */
static Dir      *Parse_dir(struct _xmlNode*);
static File     *Parse_file(struct _xmlNode*);
static Disk     *Parse_disk(struct _xmlNode*);
static DiskList *Parse_disk_list(struct _xmlNode*);

static
char * child_content(struct _xmlNode *node){
    return (node->children->content);
}

static
File * Parse_file(struct _xmlNode *node){
    File        *file = NULL;

    file = calloc(sizeof(File),sizeof(char));
    assert(file);

    file->file_type = IS_FILE;

    do{
        if(node->type == XML_ELEMENT_NODE){
           if(!strcmp(node->name,"file_name"))
               strcpy(file->file_name,child_content(node));
           else if(!strcmp(node->name,"file_size"))
               sscanf(child_content(node),"%llu",&file->file_size);
        }
    }while((node=node->next));
    return file;
}

static
Dir  * Parse_dir(struct _xmlNode *node){
    Dir         *dir  = NULL;
    File        *file = NULL;

    dir = calloc(sizeof(Dir),sizeof(char));
    assert(dir);

    dir->file_type = IS_DIRECTORY;

    do{
        if(node->type == XML_ELEMENT_NODE){
            if(!strcmp(node->name,"dir_name"))
                strcpy(dir->file_name,child_content(node));
            else if(!strcmp(node->name,"file")){
                file = Parse_file(node->children);
                dir->dir_content = g_slist_append(dir->dir_content,file);
                dir->file_size+=file->file_size;
            }
        }
    }while((node=node->next));
    
    return dir;
}


static
Disk * Parse_disk(struct _xmlNode *node){
    Disk        *disk = NULL;
    File        *file = NULL;
    Dir         *dir  = NULL;

    disk = calloc(sizeof(Disk),sizeof(char));
    assert(disk);

    do{
        if(node->type == XML_ELEMENT_NODE){
            if(!strcmp(node->name,"disk_name"))
                strcpy(disk->disk_name,child_content(node));
            else if(!strcmp(node->name,"disk_desc")){
                int  len;
                char *desc;

                desc = child_content(node);
                len  = strlen(desc);

                disk->disk_desc = calloc(len+1,sizeof(char));

                strcpy(disk->disk_desc,child_content(node));
            }
            else if(!strcmp(node->name,"file")){
                file = Parse_file(node->children);
                disk->disk_content = g_slist_append(disk->disk_content,file);
                disk->disk_size+=file->file_size;
            }
            else if(!strcmp(node->name,"directory")){
                dir  = Parse_dir(node->children);
                disk->disk_content = g_slist_append(disk->disk_content,dir);
                disk->disk_size+=dir->file_size;
            }
        }
    }while((node=node->next));

    return disk;
}

static 
DiskList *Parse_disk_list(struct _xmlNode *node){
    DiskList    *dl    = NULL;
    Disk        *disk  = NULL;       


    dl  = calloc(sizeof(DiskList),sizeof(char));

    do{
        if(node->type == XML_ELEMENT_NODE){
            disk = Parse_disk(node->children);
            dl->disk_list = g_slist_append(dl->disk_list,disk);
            dl->total_size+=disk->disk_size;
        }
    }while((node=node->next));

    return dl;
}

DiskList *get_disk_list(char *filename) 
{
    xmlDocPtr   doc;
    DiskList    *dl;

    doc = xmlReadFile(filename, NULL, 0);

    dl  = Parse_disk_list(doc->children->children);

    if (doc == NULL) {
        fprintf(stderr, "Failed to parse %s\n", filename);
        return NULL;
    }
    
    xmlFreeDoc(doc);
    xmlCleanupParser();
    xmlMemoryDump();

    return dl;
}
