#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include "util.h"

#include "disk_info.h"

#include "win.h"
#include "win_add.h"
#include "menu_uim.h"


/* 
 * Window callback
 */
static gboolean delete_event(GtkWidget *w,GdkEvent *e,gpointer data){
    return FALSE;
}

static void destroy(GtkWidget *widget,gpointer data ){
    gtk_main_quit ();
}


static 
GtkTreeModel * create_model (void){
    GtkTreeStore *model;

    model = gtk_tree_store_new (NUM_COLUMNS,
                                G_TYPE_STRING, //DISK_COLUMN
                                G_TYPE_STRING, //SIZE_COLUMN
                                G_TYPE_STRING, //KING_COLUMN
                                G_TYPE_STRING, //DATE
                                G_TYPE_POINTER);
    return GTK_TREE_MODEL (model);
}

static
void add_columns (GtkTreeView *treeview,char *text,int col_id)
{
    GtkCellRenderer     *renderer;
    GtkTreeModel        *model;
    gint                col_offset;
    GtkTreeViewColumn   *column;
        
    model    = gtk_tree_view_get_model (treeview);

    renderer = gtk_cell_renderer_text_new (); 

    col_offset = gtk_tree_view_insert_column_with_attributes(
                           GTK_TREE_VIEW (treeview),
                           -1,text, renderer,
                          "text",col_id,
                           NULL);

    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview),col_offset-1);
    gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN (column), TRUE);
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN (column), TRUE);
    gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),col_id);

}
void win_init(int *argc,char ***argv)
{
    GtkWidget *window           = NULL;
    GtkWidget *win_v_cont       = NULL;
    GtkWidget *scroll_win       = NULL;
    GtkWidget *treeview         = NULL;
    GtkWidget *status_bar       = NULL;

    /* init jeu*/
    gtk_init(argc,argv);

    /* La fenetre principale */
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    gtk_window_set_title(GTK_WINDOW (window), "Disk Tracker");
    gtk_window_resize(GTK_WINDOW(window),772,464);

    g_signal_connect (G_OBJECT (window), "delete_event",
                      G_CALLBACK(delete_event), NULL);
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (destroy), NULL);

    gtk_container_set_border_width (GTK_CONTAINER(window), 0);


    /* 
     * Main vertical container
     */
    win_v_cont = gtk_vbox_new(FALSE,0);
    g_object_set_data (G_OBJECT(window),"win_v_cont",win_v_cont);
    gtk_container_add(GTK_CONTAINER(window),win_v_cont);

    /* 
     * Menu
     */
    do_menu(window);

    /*
     * Container de la partie application
     */

    /* La scrollbar */
    scroll_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scroll_win),
                                         GTK_SHADOW_ETCHED_IN);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_win),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(win_v_cont),scroll_win, TRUE, TRUE, 0);


    /* Le tree view */
    treeview = gtk_tree_view_new ();
    g_object_set_data (G_OBJECT(window),"treeview",treeview);

    gtk_tree_view_set_model(GTK_TREE_VIEW (treeview),create_model());
    /* gris blanc gris blanc ... */
    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (treeview), TRUE);
    gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW (treeview), TRUE);
    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview), TRUE);
    /* Peut selectionner plusiers */
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW
                                 (treeview)), GTK_SELECTION_MULTIPLE);

    gtk_container_add (GTK_CONTAINER (scroll_win),treeview); 

    add_columns(GTK_TREE_VIEW(treeview),"Disk",COLUMN_NAME);
    add_columns(GTK_TREE_VIEW(treeview),"Size",COLUMN_SIZE);
    add_columns(GTK_TREE_VIEW(treeview),"Kind",COLUMN_KIND);
    add_columns(GTK_TREE_VIEW(treeview),"Creation Date",COLUMN_DATE);

    /* 
     * Le status bar
     */ 
    status_bar = gtk_statusbar_new();
    g_object_set_data (G_OBJECT(window),"status_bar",status_bar);
    gtk_box_pack_start(GTK_BOX(win_v_cont),status_bar,FALSE,TRUE,0);

    gtk_widget_show_all(window);
    gtk_main();
}
